from redis import StrictRedis

from .base import ConfigStoreBase, ConfigurationNotFound


class RedisStore(ConfigStoreBase):
    __slots__ = ["redis"]

    def __init__(self, redis: StrictRedis):
        """Initialize the Redis configuration store

        :param redis: Handle to the Redis connection to use to retrieve
                      configuration from
        :type redis: StrictRedis
        """

        self.redis = redis

    def retrieve(self, name: str) -> str:
        """Retrieve configuration from Redis

        :param name: Name of the configuration to retrieve
        :param name: str
        :raises ConfigurationNotFound: if the named configuration can't be found
        :return: The configuration "file" contents
        :rtype: str
        """

        super().retrieve(name)

        config = self.redis.get("saas:settings:" + name)

        if not config:
            raise ConfigurationNotFound

        return config.decode("utf-8")



from .base import ConfigStoreBase, ConfigurationNotFound
from .filestore import FileStore
from .redis import RedisStore

__all__ = [
    "ConfigStoreBase",
    "FileStore",
    "RedisStore",
    "ConfigurationNotFound",
]



import os

from .base import ConfigStoreBase


class FileStore(ConfigStoreBase):
    __slots__ = ["base_directory"]

    def __init__(self, directory=None):
        """Initialize connection to filestore

        :param directory: location of configuration files
        :type directory: str
        """

        self.base_directory = directory

    def retrieve(self, name: str = None) -> str:
        """Retrieve configuration from a file

        :param name: base name of the configuration file
        :type name: str
        :return: configuration file contents
        :rtype: str
        """

        super().retrieve(name)

        fullpath = os.path.join(self.base_directory, name + ".conf")

        with open(fullpath, "r", encoding="utf-8") as file:
            content = file.read()

        return content



from apacheconfig import make_loader

from .base import ConfigParserBase


class ApacheConfigParser(ConfigParserBase):
    __slots__ = ["content"]

    def parse(self, content: str) -> dict:
        """Return a dict with the parsed config

        :return: Parsed configuration
        :rtype: dict
        """

        content = super().parse(content)

        if content == "":
            raise ValueError("Cannot parse empty configuration")

        config = None
        with make_loader() as loader:
            config = loader.loads(content)

        return config



import re
from abc import ABC, abstractmethod


class ConfigurationNotFound(FileNotFoundError):
    """Exception raised when configuration can't be found
    """

    pass


class ConfigStoreBase(ABC):
    """Abstract base class for configuration store plugins
    """

    ALLOWED_CONFIG_NAME = r"^[a-zA-Z0-9_-]+(\.[a-zA-Z0-9_-]+)*$"

    @abstractmethod
    def retrieve(self, name: str):
        """Retrieve configuration - base/superclass method

        :param name: Name of the configuration to retrieve
        :param name: str
        :raises NameError: If the configuration name is invalid
        """

        if not re.match(self.ALLOWED_CONFIG_NAME, name):
            raise NameError



from minty_config.parser.base import ConfigParserBase
from minty_config.store.base import ConfigStoreBase


class Configuration:
    __slots__ = ["parser", "store"]

    def __init__(self, store: ConfigStoreBase, parser: ConfigParserBase):
        """Initialize a configuration object

        :param store: instance of the ConfigStoreBase to use
        :type store: ConfigStoreBase
        :param parser: instance of the ConfigParserBase module to use
        :type parser: ConfigParserBase
        """

        self.parser = parser
        self.store = store

    def get(self, name: str) -> dict:
        """Get the named configuration from the configured store

        :param name: name of the environment to retrieve configuration for
        :param name: str, optional
        :return: the configuration
        :rtype: dict
        """

        raw_configuration = self.store.retrieve(name=name)
        configuration = self.parser.parse(content=raw_configuration)

        return configuration



from abc import ABC, abstractmethod


class ConfigParserBase(ABC):
    @abstractmethod
    def parse(self, content: str) -> str:
        """Parse configuration content

        :param content: contents of the configuration file to parse
        :type content: str
        :return: the "content" parameter, unchanged
        :rtype: str
        """

        return content
